from __future__ import absolute_import
from __future__ import division
from __future__ import print_function
from __future__ import unicode_literals

import json
import logging

from dba.lib import log_lib
import fbjbod


def json_print(obj):
    print(json.dumps(obj, indent=2))


def _enclosure_method_dict(method_name, method_kwargs=None):
    m_kwargs = method_kwargs if method_kwargs else {}
    return {enclosure.sg_device: getattr(enclosure, method_name)(**m_kwargs)
            for enclosure in fbjbod.Enclosure.enclosures()}


def _enclosure_method_list(method_name, list_method_name,
                           method_kwargs=None, list_method_kwargs=None):
    m_kwargs = method_kwargs if method_kwargs else {}
    l_kwargs = list_method_kwargs if list_method_kwargs else {}
    return {enclosure.sg_device: [getattr(i, list_method_name)(**l_kwargs)
                                  for i in
                                  getattr(enclosure, method_name)(**m_kwargs)]
            for enclosure in fbjbod.Enclosure.enclosures()}


def gpio_status():
    return _enclosure_method_list("_gpio_status", "_asdict")


def fan_status():
    return _enclosure_method_dict("fan_status")


def event_status():
    return _enclosure_method_dict("event_status")


def led_status():
    return _enclosure_method_list("led_status", "_asdict")


def list_hdds():
    return _enclosure_method_list("hdds", "_json")


def fault_enable_all_hdds():
    return _enclosure_method_list("hdds", "fault_enabled",
                                  list_method_kwargs={"attempts": 3})


def fault_disable_all_hdds():
    return _enclosure_method_list("hdds", "fault_disabled",
                                  list_method_kwargs={"attempts": 3})


def power_on_all_hdds():
    return _enclosure_method_list("hdds", "power_on",
                                  list_method_kwargs={"attempts": 3})


def power_off_all_hdds():
    return _enclosure_method_list("hdds", "power_off",
                                  list_method_kwargs={"attempts": 3})


def error_counters():
    return _enclosure_method_list("error_counters", "_asdict")


def list_sensors():
    return {enclosure.sg_device: {type_name: [st._json() for st in sts]
                                  for type_name, sts in
                                  enclosure.sensors().iteritems()}
            for enclosure in fbjbod.Enclosure.enclosures()}


def event_log():
    return _enclosure_method_list("event_log", "_asdict")


def main():
    import argparse
    cmds = {
        "gpio": gpio_status,
        "fans": fan_status,
        "leds": led_status,
        "event-status": event_status,
        "event-log": event_log,
        "error-counters": error_counters,
        "sensors": list_sensors,
        "hdds": list_hdds,
        "faults-enable": fault_enable_all_hdds,
        "faults-disable": fault_disable_all_hdds,
        "power-on": power_on_all_hdds,
        "power-off": power_off_all_hdds,
    }

    parser = argparse.ArgumentParser()
    parser.add_argument('cmd', choices=cmds.keys())
    log_lib.add_log_level_option(parser)
    args = parser.parse_args()

    logging.basicConfig(level=args.log_level)

    json_print(cmds[args.cmd]())


if __name__ == "__main__":
    main()
